@@ -232,7 +232,6 @@ class FunctionalEnrichment(object):
             index.extend([RES.index[i]]*len(l))
         
         RES = pd.DataFrame(index = index,data = data,columns = ['GO'])
-        
         genes = np.array(list(RES.index))
         agt = np.array(list(RES['GO'].values))
         idx = np.argsort(agt)
@@ -330,17 +329,18 @@ class FunctionalEnrichment(object):
                 ix = np.where(np.in1d(q(RES.index),gi))[0]
                 res = RES.iloc[ix]
                 goterms = np.unique(q(res['GO']))
-                goterms = goterms[goterms!='S']  
-                result = GOEA(gi,GENE_SETS,goterms=goterms,fdr_thresh=100,p_thresh=100)
+                goterms = goterms[goterms!='S'].flatten()
+                if goterms.size > 0:
+                    result = GOEA(gi,GENE_SETS,goterms=goterms,fdr_thresh=100,p_thresh=100)
 
-                lens = np.array([len(np.unique(x.split(';'))) for x in result['genes'].values])
-                F = -np.log10(result['p_value'])
-                gt,vals = F.index,F.values
-                Z = pd.DataFrame(data=np.arange(CAT_NAMES.size)[None,:],columns=CAT_NAMES)
-                if gt.size>0:
-                    HM[Z[gt].values.flatten(),ii] = vals
-                    HMe[Z[gt].values.flatten(),ii] = lens
-                    HMg[Z[gt].values.flatten(),ii] = [';'.join(np.unique(x.split(';'))) for x in result['genes'].values]
+                    lens = np.array([len(np.unique(x.split(';'))) for x in result['genes'].values])
+                    F = -np.log10(result['p_value'])
+                    gt,vals = F.index,F.values
+                    Z = pd.DataFrame(data=np.arange(CAT_NAMES.size)[None,:],columns=CAT_NAMES)
+                    if gt.size>0:
+                        HM[Z[gt].values.flatten(),ii] = vals
+                        HMe[Z[gt].values.flatten(),ii] = lens
+                        HMg[Z[gt].values.flatten(),ii] = [';'.join(np.unique(x.split(';'))) for x in result['genes'].values]
 
         #CAT_NAMES = [_KOG_TABLE[x] for x in CAT_NAMES]
         SC = pd.DataFrame(data = HM,index=CAT_NAMES,columns=all_nodes).T
@@ -686,8 +686,17 @@ class GenePairFinder(object):
             b = '_'.join(ct2[i].split('_')[1:])
             print('Calculating gene pairs for the mapping: {};{} to {};{}'.format(ct1[i].split('_')[0],a,ct2[i].split('_')[0],b))
             res['{};{}'.format(ct1[i],ct2[i])] = self.find_genes(ct1[i],ct2[i],**kwargs)
-            
-        res = pd.DataFrame([res[k][0] for k in res.keys()],index=res.keys()).fillna(np.nan).T            
+        
+        cols = []
+        col_names = []
+        for k in res:
+            col_names.append(k)
+            col_names.append(k+"_pval1")
+            col_names.append(k+"_pval2")
+            cols.append(res[k][0])
+            cols.append(res[k][-2])
+            cols.append(res[k][-1])
+        res = pd.DataFrame(cols,index=col_names).fillna(np.nan).T            
         return res
         
     def find_genes(
@@ -721,6 +730,8 @@ class GenePairFinder(object):
         G - Enriched gene pairs
         G1 - Genes from species 1 involved in enriched gene pairs
         G2 - Genes from species 2 involved in enriched gene pairs
+        pvals1 - pvalues for genes from species 1 involved in enriched gene pairs
+        pvals2 - pvalues for genes from species 2 involved in enriched gene pairs
         """
         n1 = str(n1)
         n2 = str(n2)
@@ -738,13 +749,11 @@ class GenePairFinder(object):
         G = q(gpairs[np.argsort(-m)[:n_genes]])
         G1 = substr(G, ";", 0)
         G2 = substr(G, ";", 1)
+        pvals1 = q(sam1.adata.varm[self.keys[id1] + "_pvals"][n1][G1])
+        pvals2 = q(sam2.adata.varm[self.keys[id2] + "_pvals"][n2][G2])
+        filt = np.logical_and(pvals1 < thr,pvals2 < thr)
         G = q(
-            G[
-                np.logical_and(
-                    q(sam1.adata.varm[self.keys[id1] + "_pvals"][n1][G1] < thr),
-                    q(sam2.adata.varm[self.keys[id2] + "_pvals"][n2][G2] < thr),
-                )
-            ]
+            G[filt]
         )
         G1 = substr(G, ";", 0)
         G2 = substr(G, ";", 1)
@@ -752,7 +761,7 @@ class GenePairFinder(object):
         _, ix2 = np.unique(G2, return_index=True)
         G1 = G1[np.sort(ix1)]
         G2 = G2[np.sort(ix2)]
-        return G, G1, G2
+        return G, G1, G2, pvals1[filt], pvals2[filt]
 
     def _find_link_genes_avg(self, c1, c2, id1, id2, w1t=0.35, w2t=0.35, expr_thr=0.05):
         mus = self.mus
@@ -1473,21 +1482,25 @@ def _compute_csim(sam3, key, X=None, prepend=True, n_top = 0):
     valdict = df_to_dict(A, key_key="x", val_key="y")   
     cell_scores = [valdict[k].sum() for k in valdict.keys()]
     ixer = pd.Series(data=np.arange(clu.size),index=clu)
-    xc,yc = substr(list(valdict.keys()),';')
-    xc = xc.astype('int')
-    yc=ixer[yc].values
-    cell_cluster_scores = sp.sparse.coo_matrix((cell_scores,(xc,yc)),shape=(X.shape[0],clu.size)).A
+    if len(valdict.keys())>0:
+        xc,yc = substr(list(valdict.keys()),';')
+        xc = xc.astype('int')
+        yc=ixer[yc].values
+        cell_cluster_scores = sp.sparse.coo_matrix((cell_scores,(xc,yc)),shape=(X.shape[0],clu.size)).A
 
-    for i, c in enumerate(clu):
-        if n_top > 0:
-            CSIM[i, :] = np.sort(cell_cluster_scores[cl==c],axis=0)[-n_top:].mean(0)
-        else:
-            CSIM[i, :] = cell_cluster_scores[cl==c].mean(0)
+        for i, c in enumerate(clu):
+            if n_top > 0:
+                CSIM[i, :] = np.sort(cell_cluster_scores[cl==c],axis=0)[-n_top:].mean(0)
+            else:
+                CSIM[i, :] = cell_cluster_scores[cl==c].mean(0)
 
-    CSIM = np.stack((CSIM,CSIM.T),axis=2).max(2)
-    CSIMth = CSIM / sam3.adata.obsp['knn'][0].data.size * (len(skeys)-1)
-    return CSIMth,clu
+        CSIM = np.stack((CSIM,CSIM.T),axis=2).max(2)
+        CSIMth = CSIM / sam3.adata.obsp['knn'][0].data.size * (len(skeys)-1)
+        return CSIMth,clu
+    else:
+        return np.zeros((clu.size, clu.size)), clu
 
+# TODO: Deprecate for now and fix this later
 def transfer_annotations(sm,reference_id=None, keys=[],num_iters=5, inplace = True):
     """ Transfer annotations across species using label propagation along the combined manifold.
     
@@ -1531,8 +1544,8 @@ def transfer_annotations(sm,reference_id=None, keys=[],num_iters=5, inplace = Tr
         samref = sm.sams[reference_id]
         ANN = stitched.adata.obs
         ANNr = samref.adata.obs
-        cl = ANN[key].values.astype('object').astype('str')
-        clr = reference_id+'_'+ANNr[key].values.astype('object')
+        cl = ANN[reference_id+'_'+key].values.astype('object').astype('str')
+        clr = ANNr[key].values.astype('object')
         cl[np.invert(np.in1d(cl,clr))]=''
         clu,clui = np.unique(cl,return_inverse=True)
         P = np.zeros((NNM.shape[0],clu.size))
